<?php

namespace Drupal\skpr_key\Plugin\KeyProvider;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\key\KeyInterface;
use Drupal\key\Plugin\KeyProviderBase;
use Drupal\key\Plugin\KeyPluginFormInterface;

/**
 * Adds a key provider that allows a key to be loaded from Skpr config.
 *
 * @KeyProvider(
 *   id = "skpr",
 *   label = "Skpr",
 *   description = @Translation("This provider retrieves the key from Skpr config."),
 *   storage_method = "skpr",
 *   key_value = {
 *     "accepted" = FALSE,
 *     "required" = FALSE
 *   }
 * )
 */
class SkprKeyProvider extends KeyProviderBase implements KeyPluginFormInterface {

  /**
   * The Skpr config object.
   *
   * @var \Skpr\SkprConfig
   */
  protected $skpr;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    /** @var self $instance */
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->skpr = $container->get('skpr_key.skpr_config_factory')->getInstance();
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'skpr_key' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $options = [];
    foreach ($this->skpr->getAll() as $key => $config) {
      $options[$key] = $key;
    }

    $form['skpr_key'] = [
      '#type' => 'select',
      '#title' => $this->t('Skpr key'),
      '#description' => $this->t('Name of the skpr config to load.'),
      '#required' => TRUE,
      '#options' => $options,
      '#empty_value' => '',
      '#default_value' => $this->getConfiguration()['skpr_key'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $key_provider_settings = $form_state->getValues();
    $skpr_key = $key_provider_settings['skpr_key'];

    // Does the config exist.
    $key_value = $this->skpr->get($skpr_key, '');
    if (empty($key_value)) {
      $form_state->setErrorByName('skpr_key', $this->t('The key does not exist or it is empty.'));
      return;
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->setConfiguration($form_state->getValues());
  }

  /**
   * {@inheritdoc}
   */
  public function getKeyValue(KeyInterface $key) {
    $skpr_key = $this->configuration['skpr_key'];
    $key_value = $this->skpr->get($skpr_key, '');

    if (!$key_value) {
      return NULL;
    }

    return $key_value;
  }

}
